==> hestia_earth_extend_bibliography-0.3.1-py3-none-any.whl: hestia_earth/extend_bibliography/bibliography_apis/crossref.py <==
import json
import traceback
from concurrent.futures import ThreadPoolExecutor
import habanero
from hestia_earth.schema import Bibliography

from hestia_earth.extend_bibliography.log import logger
from hestia_earth.extend_bibliography.utils import current_time_ms
from .utils import (
    ORINGAL_FIELD, MAXIMUM_DISTANCE, find_closest_result, extend_bibliography, remove_empty_values,
    capitalize, update_actor_names
)


def author_to_actor(author):
    return update_actor_names({
        'firstName': capitalize(author['given']),
        'lastName': capitalize(author['family'])
    })


def clean_abstract(abstract: str):
    return abstract.replace('<jats:p>', '').replace('</jats:p>', '').strip() if abstract else None


def item_to_bibliography(item):
    outlet = item.get('short-container-title')
    return {
        'title': capitalize(item.get('title')[0]),
        'year': item.get('issued').get('date-parts')[0][0],
        'outlet': capitalize(outlet[0]) if outlet and len(outlet) > 0 else None,
        'abstract': clean_abstract(item.get('abstract')),
        'documentDOI': item.get('DOI')
    }


def create_biblio(title: str, item: dict):
    biblio = Bibliography()
    # save title here since closest item might differ
    biblio.fields[ORINGAL_FIELD + 'title'] = title
    biblio.fields['title'] = title
    authors = list(map(author_to_actor, item['author'] if item else []))
    bibliography = item_to_bibliography(item) if item else {}
    (extended_biblio, actors) = extend_bibliography(authors, bibliography['year']) if item else ({}, [])
    return (
        {**biblio.to_dict(), **bibliography, **extended_biblio},
        actors
    ) if item else (biblio.to_dict(), [])


def exec_search(cr):
    def search(title: str):
        items = cr.works(query=title)['message']['items']
        return list(map(lambda x: {'title': x['title'][0], 'item': x}, items))
    return search


def search(cr, title):
    [item, distance] = find_closest_result(title, exec_search(cr))
    return create_biblio(title, item if distance <= MAXIMUM_DISTANCE else None)


def extend_title(cr, bibliographies, actors):
    def extend(title: str):
        now = current_time_ms()
        (biblio, authors) = search(cr, title)
        logger.debug('find in %sms: %s', current_time_ms() - now, title)
        logger.debug('found bibliography: %s', json.dumps(biblio, indent=2))
        bibliographies.extend([] if biblio is None else [biblio])
        actors.extend([] if authors is None else authors)
    return extend


def extend_crossref(titles, **kwargs):
    try:
        cr = habanero.Crossref()

        bibliographies = []
        actors = []

        extender = extend_title(cr, bibliographies, actors)
        with ThreadPoolExecutor() as executor:
            executor.map(extender, titles)

        return (remove_empty_values(actors), remove_empty_values(bibliographies))
    except Exception as e:
        logger.error(str(e))
        logger.error(traceback.format_exc())
        return ([], [])

==> hestia_earth_extend_bibliography-0.4.0-py3-none-any.whl: hestia_earth/extend_bibliography/bibliography_apis/crossref.py <==
import json
import traceback
from concurrent.futures import ThreadPoolExecutor
import habanero
from hestia_earth.schema import Bibliography, SourceLicense

from hestia_earth.extend_bibliography.log import logger
from hestia_earth.extend_bibliography.utils import current_time_ms
from .utils import (
    ORINGAL_FIELD, MAXIMUM_DISTANCE, find_closest_result, extend_bibliography, remove_empty_values,
    capitalize, update_actor_names, update_node_value
)


def _author_to_actor(author):
    return update_actor_names({
        'firstName': capitalize(author['given']),
        'lastName': capitalize(author['family'])
    })


def _clean_abstract(abstract: str):
    return abstract.replace('<jats:p>', '').replace('</jats:p>', '').strip() if abstract else None


def _extract_cc_license(url: str):
    suffix = url.replace('https://creativecommons.org/licenses/', '').split('/')[0]
    return '-'.join(['cc', suffix]).upper()


def _parse_license(licenses: list):
    license_url = licenses[0].get('URL') if len(licenses) else None
    return None if license_url is None else (
        _extract_cc_license(license_url) if license_url.startswith('https://creativecommons.org/licenses') else
        SourceLicense.OTHER_LICENSE.value
    )


def item_to_bibliography(item: dict, include_license: bool = False):
    outlet = item.get('short-container-title')
    return {
        'title': capitalize(item.get('title')[0]),
        'year': item.get('issued').get('date-parts')[0][0],
        'outlet': capitalize(outlet[0]) if outlet and len(outlet) > 0 else None,
        'abstract': _clean_abstract(item.get('abstract')),
        'documentDOI': item.get('DOI'),
        'license': _parse_license(item.get('license', [])) if include_license else None
    }


def create_biblio(title: str, item: dict):
    biblio = Bibliography()
    # save title here since closest item might differ
    biblio.fields[ORINGAL_FIELD + 'title'] = title
    biblio.fields['title'] = title
    authors = list(map(_author_to_actor, item['author'] if item else []))
    bibliography = item_to_bibliography(item) if item else {}
    (extended_biblio, actors) = extend_bibliography(authors, bibliography['year']) if item else ({}, [])
    return (
        {**biblio.to_dict(), **bibliography, **extended_biblio},
        actors
    ) if item else (biblio.to_dict(), [])


def exec_search(cr):
    def search(title: str):
        items = cr.works(query=title)['message']['items']
        return list(map(lambda x: {'title': x['title'][0], 'item': x}, items))
    return search


def search(cr, title):
    [item, distance] = find_closest_result(title, exec_search(cr))
    return create_biblio(title, item if distance <= MAXIMUM_DISTANCE else None)


def extend_title(cr, bibliographies, actors):
    def extend(title: str):
        now = current_time_ms()
        (biblio, authors) = search(cr, title)
        logger.debug('find in %sms: %s', current_time_ms() - now, title)
        logger.debug('found bibliography: %s', json.dumps(biblio, indent=2))
        bibliographies.extend([] if biblio is None else [biblio])
        actors.extend([] if authors is None else authors)
    return extend


def _search_by_documentDOI(value: str):
    try:
        cr = habanero.Crossref()
        data = cr.works(ids=value.strip())['message']
        return item_to_bibliography(data, True)
    except Exception:
        return {}


def extend_source_license(source: dict) -> dict:
    key = 'license'
    doi = source.get('bibliography', {}).get('documentDOI')
    run_extend = doi is not None and source.get(key, None) is None
    item = _search_by_documentDOI(doi) if run_extend else {}
    return update_node_value(source, key, item.get(key))


def extend_crossref(titles, **kwargs):
    try:
        cr = habanero.Crossref()

        bibliographies = []
        actors = []

        extender = extend_title(cr, bibliographies, actors)
        with ThreadPoolExecutor() as executor:
            executor.map(extender, titles)

        return (remove_empty_values(actors), remove_empty_values(bibliographies))
    except Exception as e:
        logger.error(str(e))
        logger.error(traceback.format_exc())
        return ([], [])
